###Author: Melissa May Maestas

# Functions related to merging data

# merge predictor variables together
merge_predictors.fn <- function(X) { #(predictand_data,predictand_col,latitude_col_t,longitude_col_t,datum_col_t, Easting_col_t, Northing_col_t,Dates_col_t, output_file_name, output_sub_folder, study_start_date, study_stop_date) {
  #### Define Date to be processed in this iteration and file name to be output ####
  this_Date <- as.Date(Date_list[X]) # identify the date for this iteration
  print(this_Date) # COMMENT
  this_file <- file.path(ProcessedData.directory,output_sub_folder,output_sub_sub_folder,paste(ML_input_file_name_output,"_",this_Date,'.csv',sep = "")) # name of file to be output
  #print(this_file) 
  #file.exists(this_file) # COMMENT
  #### Check if output file already exists and process data if it doesn't exist ####
  if (file.exists(this_file)) { # only run code if file doesn't already exist
    print(this_file)
    print("already exists")
  } else {
    #### Define columns and change column names as needed. Check how many rows there are in ML_input - this should not change for the rest of this function ####
    print("file does not already exist - need to generate file")
    which_this_date <- which(Source_Data[ ,Dates_col_t] == this_Date) # identify all data for this date
    if (predictand_col %in% colnames(Source_Data)) { # this data includes PM2.5 data (training vs prediction data sets)
      vars_to_include <- c(predictand_col,latitude_col_t,longitude_col_t,Dates_col_t,"Year","Month","Day") # which variables to keep # datum_col_t,
    } else { # if (predictand_col %in% colnames(Source_Data)) { # this data includes PM2.5 data
      Source_Data$Year <- input_mat_extract_year_from_date.fn(Source_Data[ ,c(Dates_col_t)]) # add Year column
      Source_Data$Month <- input_mat_extract_month_from_date.fn(Source_Data[ ,c(Dates_col_t)]) # add month column
      Source_Data$Day <- input_mat_extract_day_from_date.fn(Source_Data[ ,c(Dates_col_t)]) # add Day column
      vars_to_include <- c(latitude_col_t,longitude_col_t,Dates_col_t,"Year","Month","Day") # which variables to keep
    } # if (predictand_col %in% colnames(Source_Data)) { # this data includes PM2.5 data
    ML_input_step <- Source_Data[which_this_date, vars_to_include] # start ML_input data frame
    ML_input <- ML_input_step[!duplicated(ML_input_step), ] # de-duplicate rows of data
    rm(ML_input_step) # clear variable
    n_rows <- dim(ML_input)[1] # determine number of rows (should not change within function)
    n_rows_orig <- n_rows # define number of rows variable
    n_cols_orig <- dim(ML_input)[2] # determine number of columns - used to make sure this is changing correctly
    ML_input <- replace_column_names.fn(df_in = ML_input, old_col_name = "Lat", new_col_name = "Latitude") # replace "Lat" with "Latitude"
    ML_input <- replace_column_names.fn(df_in = ML_input, old_col_name = "Lon", new_col_name = "Longitude") # replace "Lat" with "Latitude"
    ML_input <- replace_column_names.fn(df_in = ML_input, old_col_name = "Date_Local", new_col_name = "Date") # replace "Lat" with "Latitude"
    rm(which_this_date) # clear variable
    if (n_rows != dim(ML_input)[1]) {stop("Number of rows in ML_input is changing, line 22")} # error message to check data
    added_cols <- 0 # start counter for columns
  
    #### Merge MODIS Active Fire Points, with lags ####
      for (this_lag in 0:7) { # cycle through lag days and add column for each
        print(paste("this lag =",this_lag)) # COMMENT
        # Load and merge Fire MODIS 25 km Data and its lags
          print(paste("merge 25 km Fire data for lag",this_lag))
          ML_input <- merge_Fire_MODIS_data.fn(Buffer_radius_km = 25, ML_input = ML_input, Fire_MODIS_file_name = fire_MODIS_25km_file_name,
                                               ProcessedData.directory = ProcessedData.directory, predictor_sub_folder = predictor_sub_folder,
                                               this_Date = this_Date, lag_n_days = this_lag, no_match_value = 0)
          if (n_rows != dim(ML_input)[1]) {stop(paste("***Number of rows in ML_input is changing with after merging 25 km Fire MODIS data when processing lag_day =",this_lag,"***"))}
          additional_cols <- 1
          added_cols <- added_cols+additional_cols
          if (dim(ML_input)[2] != (n_cols_orig+added_cols)) {stop(paste("Check number of columns after merging 25 km Fire MODIS data when processing lag_day =",this_lag,"***"))}
        # Load and merge Fire MODIS 50 km Data
          print(paste("merge 50 km Fire data for lag",this_lag))
        ML_input <- merge_Fire_MODIS_data.fn(Buffer_radius_km = 50, ML_input = ML_input, Fire_MODIS_file_name = fire_MODIS_50km_file_name,
                                             ProcessedData.directory = ProcessedData.directory, predictor_sub_folder = predictor_sub_folder,
                                             this_Date = this_Date, lag_n_days = this_lag, no_match_value = 0)
        if (n_rows != dim(ML_input)[1]) {stop(paste("Number of rows in ML_input is changing after merging 50 km Fire MODIS data when processing lag_day =",this_lag,"***"))}
        additional_cols <- 1
        added_cols <- added_cols+additional_cols
        if (dim(ML_input)[2] != (n_cols_orig+added_cols)) {stop(paste("Check number of columns after merging 50 km Fire MODIS data when processing lag_day =",this_lag,"***"))}

        #print("UNCOMMENT CODE FOR 100 KM ACTIVE FIRE POINTS")
        # Load and merge Fire MODIS 100 km Data
        print(paste("merge 100 km Fire data for lag",this_lag))
        ML_input <- merge_Fire_MODIS_data.fn(Buffer_radius_km = 100, ML_input = ML_input, Fire_MODIS_file_name = fire_MODIS_100km_file_name,
                                             ProcessedData.directory = ProcessedData.directory, predictor_sub_folder = predictor_sub_folder,
                                             this_Date = this_Date, lag_n_days = this_lag, no_match_value = 0)
        if (n_rows != dim(ML_input)[1]) {stop(paste("Number of rows in ML_input is changing after merging 100 km Fire MODIS data when processing lag_day =",this_lag,"***"))}
        additional_cols <- 1
        added_cols <- added_cols+additional_cols
        if (dim(ML_input)[2] != (n_cols_orig+added_cols)) {stop(paste("Check number of columns after merging 100 km Fire MODIS data when processing lag_day =",this_lag,"***"))}

        #print("UNCOMMENT CODE FOR 500 KM ACTIVE FIRE POINTS")
        # Load and merge Fire MODIS 500 km Data
        ML_input <- merge_Fire_MODIS_data.fn(Buffer_radius_km = 500, ML_input = ML_input, Fire_MODIS_file_name = fire_MODIS_500km_file_name,
                                             ProcessedData.directory = ProcessedData.directory, predictor_sub_folder = predictor_sub_folder,
                                             this_Date = this_Date, lag_n_days = this_lag, no_match_value = 0)
        if (n_rows != dim(ML_input)[1]) {stop(paste("Number of rows in ML_input is changing after merging 500 km Fire MODIS when processing lag_day =",this_lag,"***"))}
        additional_cols <- 1
        added_cols <- added_cols+additional_cols
        if (dim(ML_input)[2] != (n_cols_orig+added_cols)) {stop(paste("Check number of columns after merging 500 km Fire MODIS data when processing lag_day =",this_lag,"***"))}
      } # for (this_lag in 0:7) { # cycle through lag days and add column for each
      print("finished processing active fire points") # COMMENT

    #### Determine whether there's a fire (any of the active fire variables > 0) ####
      ML_input$Binary_Fire <- 0 # create Binary_Fire column and fill with zeros
      ML_input[which(rowSums(ML_input[ ,sapply(names(ML_input), FUN = function(x){str_detect(x, "Fire_Count")})]) > 0), "Binary_Fire"] <- 1 # switch Binary_Fire to 1 for any rows that have any Fire_Count* columns greater than 0
      if (n_rows != dim(ML_input)[1]) {stop(paste("Number of rows in ML_input is changing after creating Binary_Fire column ***"))}
      additional_cols <- 1
      added_cols <- added_cols+additional_cols
      if (dim(ML_input)[2] != (n_cols_orig+added_cols)) {stop(paste("Check number of columns after creating Binary_Fire column ***"))}

    #### Load and merge Highways Data ####
      print("start merging Highways data") # COMMENT
      ML_input <- merge_Highways_data.fn(ML_input = ML_input, Highways_file_name = Highways_file_name, ProcessedData.directory = define_file_paths.fn("ProcessedData.directory"),predictor_sub_folder = predictor_sub_folder)#, this_Date = this_Date) #, study_start_date = study_start_date, study_stop_date = study_stop_date)
      if (n_rows != dim(ML_input)[1]) {stop(paste("Number of rows in ML_input is changing after merging Highway data. X =",X,"Date = ",this_Date))}
      additional_cols <- 12
      added_cols <- added_cols+additional_cols
      if (dim(ML_input)[2] != (n_cols_orig+added_cols)) {stop(paste("Number of rows in ML_input is changing after merging Highway data. X =",X,"Date = ",this_Date))}
    
    #### Input Population Density ####
      print("start merging population density")
      ML_input <- merge_population_data.fn(ML_input = ML_input, Pop_density_file_name = Pop_density_file_name, ProcessedData.directory = define_file_paths.fn("ProcessedData.directory"),predictor_sub_folder = predictor_sub_folder)
      if (n_rows != dim(ML_input)[1]) {stop(paste("Number of rows in ML_input is changing after merging population data. X =",X,"Date = ",this_Date))}
      additional_cols <- 1#12
      added_cols <- added_cols+additional_cols
      if (dim(ML_input)[2] != (n_cols_orig+added_cols)) {stop(paste("Number of rows in ML_input is changing after merging population data. X =",X,"Date = ",this_Date))}
      
    #### Load and merge MAIAC Data ####
      print("start merging MAIAC data") # COMMENT
      ML_input <- merge_MAIAC_data.fn(ML_input = ML_input, MAIAC_file_name = MAIAC_file_name, ProcessedData.directory = define_file_paths.fn("ProcessedData.directory"), predictor_sub_folder = predictor_sub_folder, this_Date = this_Date)
      if (n_rows != dim(ML_input)[1]) {stop(paste("Number of rows in ML_input is changing after merging MAIAC data. X =",X,"Date = ",this_Date))}
      additional_cols <- 1
      added_cols <- added_cols+additional_cols
      if (dim(ML_input)[2] != (n_cols_orig+added_cols)) {stop(paste("Number of rows in ML_input is changing after merging MAIAC data. X =",X,"Date = ",this_Date))}
    #### Load and merge NED Data ####
      print("start merging NED data") # COMMENT
      ML_input <- merge_NED_data.fn(ML_input = ML_input, NED_file_name = NED_file_name, ProcessedData.directory = define_file_paths.fn("ProcessedData.directory"), predictor_sub_folder = predictor_sub_folder)
      if (n_rows != dim(ML_input)[1]) {stop(paste("Number of rows in ML_input is changing after merging NED data. X =",X,"Date = ",this_Date))}
      additional_cols <- 1
      added_cols <- added_cols+additional_cols
      if (dim(ML_input)[2] != (n_cols_orig+added_cols)) {stop(paste("Number of rows in ML_input is changing after merging NED data. X =",X,"Date = ",this_Date))}
    #### Load and merge NAM Data ####
      print("start merging NAM data") # COMMENT
      ML_input <- merge_NAM_data.fn(ML_input = ML_input, NAM_file_name = NAM_file_name, ProcessedData.directory = define_file_paths.fn("ProcessedData.directory"), predictor_sub_folder = predictor_sub_folder, this_Date = this_Date)
      if (n_rows != dim(ML_input)[1]) {stop(paste("***Number of rows in ML_input is changing after merging NAM data*** X =",X,"Date = ",this_Date))}
      additional_cols <- 14
      added_cols <- added_cols+additional_cols
      if (dim(ML_input)[2] != (n_cols_orig+added_cols)) {stop(paste("Check number of columns after merging NAM data X =",X,"Date = ",this_Date))}
    #### Load and merge 1 km NLCD Data ####
      print("start merging 1 km NLCD data") # COMMENT
      ML_input <- merge_NLCD_data.fn(buffer_radius = "1km", ML_input = ML_input, NLCD_file_name = NLCD_1km_file_name, ProcessedData.directory = define_file_paths.fn("ProcessedData.directory"), predictor_sub_folder = predictor_sub_folder)
      if (n_rows != dim(ML_input)[1]) {stop(paste("Number of rows in ML_input is changing after merging NLCD 1 km data. X =",X,"Date = ",this_Date))}
      additional_cols <- 1
      added_cols <- added_cols+additional_cols
      if (dim(ML_input)[2] != (n_cols_orig+added_cols)) {stop(paste("Number of rows in ML_input is changing after merging 1 km NLCD data. X =",X,"Date = ",this_Date))}
    #### Load and merge 5 km NLCD Data ####
      print("start merging 5 km NLCD data") # COMMENT
      ML_input <- merge_NLCD_data.fn(buffer_radius = "5km", ML_input = ML_input, NLCD_file_name = NLCD_5km_file_name, ProcessedData.directory = define_file_paths.fn("ProcessedData.directory"), predictor_sub_folder = predictor_sub_folder)
      if (n_rows != dim(ML_input)[1]) {stop(paste("Number of rows in ML_input is changing after merging NLCD 5 km data. X =",X,"Date = ",this_Date))}
      additional_cols <- 1
      added_cols <- added_cols+additional_cols
      if (dim(ML_input)[2] != (n_cols_orig+added_cols)) {stop(paste("Number of rows in ML_input is changing after merging 5 km NLCD data. X =",X,"Date = ",this_Date))}
    #### Load and merge 10 km NLCD Data ####
      print("start merging 10 km NLCD data") # COMMENT
      ML_input <- merge_NLCD_data.fn(buffer_radius = "10km", ML_input = ML_input, NLCD_file_name = NLCD_10km_file_name, ProcessedData.directory = define_file_paths.fn("ProcessedData.directory"), predictor_sub_folder = predictor_sub_folder)
      if (n_rows != dim(ML_input)[1]) {stop(paste("Number of rows in ML_input is changing after merging NLCD 10 km data. X =",X,"Date = ",this_Date))}
      additional_cols <- 1
      added_cols <- added_cols+additional_cols
      if (dim(ML_input)[2] != (n_cols_orig+added_cols)) {stop(paste("Number of rows in ML_input is changing after merging 5 km NLCD data. X =",X,"Date = ",this_Date))}
    #### Load and merge NDVI Data ####
      print("start merging NDVI data") # COMMENT
    #   print("*** insert NDVI function ***")
      ML_input <- merge_NDVI_data.fn(ML_input = ML_input, NDVI_file_name = NDVI_file_name, ProcessedData.directory = define_file_paths.fn("ProcessedData.directory"), predictor_sub_folder = predictor_sub_folder, this_Date = this_Date)
      if (n_rows != dim(ML_input)[1]) {stop(paste("Number of rows in ML_input is changing after merging NDVI data. X =",X,"Date = ",this_Date))}
      additional_cols <- 1
      added_cols <- added_cols+additional_cols
      if (dim(ML_input)[2] != (n_cols_orig+added_cols)) {stop(paste("Number of columns in ML_input is changing after merging NDVI data. X =",X,"Date = ",this_Date))}
    #### Add Day of Week Variable ####
      ML_input$DayOfWeek <- wday(ML_input$Date) # add day of week as predictor column
      if (n_rows != dim(ML_input)[1]) {stop(paste("Number of rows in ML_input is changing after adding DayOfWeek info. X =",X,"Date = ",this_Date))}
      additional_cols <- 1
      added_cols <- added_cols+additional_cols
      if (dim(ML_input)[2] != (n_cols_orig+added_cols)) {stop(paste("Number of columns in ML_input is changing after adding DayOfWeek. X =",X,"Date = ",this_Date))}
    #### add in season indicators ####
      ML_input <- add_season_indicator_columns.fn(df_interest = ML_input, month_col = "Month")
      if (n_rows != dim(ML_input)[1]) {stop(paste("Number of rows in ML_input is changing after adding Season indicator info. X =",X,"Date = ",this_Date))}
      additional_cols <- 4
      added_cols <- added_cols+additional_cols
      if (dim(ML_input)[2] != (n_cols_orig+added_cols)) {stop(paste("Number of columns in ML_input is changing after adding Season indicator. X =",X,"Date = ",this_Date))}
    #### input State ####
      ML_input$State<- latlong2state(as.data.frame(ML_input[,c("Longitude", "Latitude")]))
      if (n_rows != dim(ML_input)[1]) {stop(paste("Number of rows in ML_input is changing after adding State indicator info. X =",X,"Date = ",this_Date))}
      additional_cols <- 1
      added_cols <- added_cols+additional_cols
      if (dim(ML_input)[2] != (n_cols_orig+added_cols)) {stop(paste("Number of columns in ML_input is changing after adding State indicator. X =",X,"Date = ",this_Date))}
    #### Input cosine (day of year) ####  
      my_DOY<- apply(ML_input[,c("Year", "Month", "Day")], MARGIN = 1, FUN = DOY)
      ML_input$CosDOY<- cos(2*pi*my_DOY/365)
      rm(my_DOY)
      if (n_rows != dim(ML_input)[1]) {stop(paste("Number of rows in ML_input is changing after adding cosine(DayOfYear). X =",X,"Date = ",this_Date))}
      additional_cols <- 1
      added_cols <- added_cols+additional_cols
      if (dim(ML_input)[2] != (n_cols_orig+added_cols)) {stop(paste("Number of columns in ML_input is changing after adding cosine(DayOfYear). X =",X,"Date = ",this_Date))}
    #### write file for this date ####
      write.csv(ML_input,file = this_file,row.names = FALSE) # Write csv file
  } #if (file.exists(this_file)) { # only run code if file doesn't already exist
  print("bottom of merge_predictors.fn")
  #return(ML_input) # output from function
  return(NA) # output from function
} # end of merge_predictors.fn function

#Get day of year
DOY<- function(YMD){ # YMD is a vector containing year, month, day
  # function written by Ellen Considine, edited slightly by Melissa Maestas
  # originally in file 'Formatting data for testing.R'
  non_leap<- c(31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31)
  leap<- c(31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31)
  #Figure out if it is a leap year or not
  if(YMD[1] %in% c(2008, 2012, 2016)){
    nums<- leap
  }else{
    nums<- non_leap
  }
  
  if(YMD[2] == 1){
    return(YMD[3])
  }else{
    return(sum(nums[1:(YMD[2]-1)]) + YMD[3])
  }
} # end of function DOY<- function(YMD){ # YMD is a vector containing year, month, day

latlong2state <- function(pointsDF) {
  # function written by Ellen Considine
  # originally in file 'Formatting data for testing.R'
  # Prepare SpatialPolygons object with one SpatialPolygon
  # per state (plus DC, minus HI & AK)
  states <- map('state', fill=TRUE, col="transparent", plot=FALSE)
  IDs <- sapply(strsplit(states$names, ":"), function(x) x[1])
  states_sp <- map2SpatialPolygons(states, IDs=IDs,
                                   proj4string=CRS("+proj=longlat +datum=WGS84"))
  
  stateNames <- sapply(states_sp@polygons, function(x) x@ID)
  
  # Convert pointsDF to a SpatialPoints object 
  pointsSP <- SpatialPoints(pointsDF, 
                            proj4string=CRS("+proj=longlat +datum=WGS84"))
  
  # Use 'over' to get _indices_ of the Polygons object containing each point 
  indices <- over(pointsSP, states_sp)
  
  NApos<- which(is.na(indices))
  
  for(p in NApos){
    if(pointsDF[p,2]<= 41.9952){
      if(pointsDF[p,1] <= -114.1315){
        indices[p]<- which(stateNames == "california")
      }else if(pointsDF[p,1] <= -109.0475){
        indices[p]<- which(stateNames == "arizona")
      }else{
        indices[p]<- which(stateNames == "new mexico")
      }
    }else if((pointsDF[p,2] > 41.9952)& (pointsDF[p,2]<= 46.2891) ){
      indices[p]<- which(stateNames == "oregon")
    }else if(pointsDF[p,2] > 46.2891){
      if(pointsDF[p,1] <= -117.2372){
        indices[p]<- which(stateNames == "washington")
      }else if(pointsDF[p,1] <= -116.0458){
        indices[p]<- which(stateNames == "idaho")
      }else{
        indices[p]<- which(stateNames == "montana")
      }
    }
  }
  
  # Return the state names of the Polygons object containing each point
  stateNames[indices]
}

add_season_indicator_columns.fn <- function(df_interest, month_col){ # add 4 columns with binary season indicators
  
  Months_by_season <- list()
  Months_by_season[["Winter"]][["Months"]] <- c(12,1,2)
  Months_by_season[["Winter"]][["SeasonName"]] <- "Winter"
  Months_by_season[["Spring"]][["Months"]] <- c(3,4,5)
  Months_by_season[["Spring"]][["SeasonName"]] <- "Spring"
  Months_by_season[["Summer"]][["Months"]] <- c(6,7,8)
  Months_by_season[["Summer"]][["SeasonName"]] <- "Summer"
  Months_by_season[["Fall"]][["Months"]] <- c(9, 10, 11)
  Months_by_season[["Fall"]][["SeasonName"]] <- "Fall"
  
  for (season_i in 1:length(Months_by_season)) { # Cycle through columns and add season indicators
    df_interest[ , Months_by_season[[season_i]][["SeasonName"]]] <- 0
    which_in_season <- which(df_interest[ , month_col] %in% Months_by_season[[season_i]][["Months"]])
    df_interest[which_in_season, Months_by_season[[season_i]][["SeasonName"]]] <- 1
  } # for (season_i in 1:length(Months_by_season)) { # Cycle through columns and add season indicators
  
  return(df_interest)
} # end of add season_indicator_columns.fn function

# remove data points outside specified range of values
remove_data_outside_range.fn <- function(df_in, column_of_interest, upper_limit = NA, lower_limit = NA, include_upper_limit = TRUE, include_lower_limit = TRUE, remove_NAs = TRUE, verbose = TRUE, reason_removed = " ") {
  # df_in = input_mat1
  # column_of_interest = "PM2.5_Obs"
  # upper_limit = NA
  # lower_limit = 0
  # include_upper_limit = TRUE
  # include_lower_limit = TRUE
  # remove_NAs = TRUE
  # verbose = TRUE
  # reason_removed = "Remove negative and NA PM2.5"
  
  # remove NA values (step1)
  if (remove_NAs == TRUE) { # NA values should be removed
    which_not_NA <- which(is.na(df_in[ , column_of_interest]) == FALSE) # find the not-NAs
    df_step1 <- df_in[which_not_NA, ] # keep all data that doesn't have NA in the column of interest
    # print(paste((dim(df_in)[1] - dim(df_step1)[1])," data points were removed due to having ",column_of_interest," as NA ",sep = "")) # COMMENT
    # track which data points have been removed and why
    which_remove <- which(is.na(df_in[ , column_of_interest]) == TRUE) # find the NAs
    remove_df_NA <- df_in[which_remove, ]
    if (dim(remove_df_NA)[1]>1) { # input reason for data removal into data frame
      remove_df_NA$Reason <- reason_removed
    } else {
      remove_df_NA$Reason <- remove_df_NA[0,1]
    }# if (dim(remove_df_NA)[1]>1) { # input reason for data removal
    rm(which_not_NA,which_remove)
  } else { # NA values should not be removed
    df_step1 <- df_in
    remove_df_NA <- df_in[0,]
    remove_df_NA$Reason <- df_in[0,1]
  }
  
  # remove data above the upper limit (step2)
  if (is.na(upper_limit) == FALSE) { # an upper limit has been set
    if (include_lower_limit == TRUE) {# data of exactly the upper_limit will be kept
      which_in_range <- which(df_step1[ , column_of_interest] <= upper_limit) # find the data at or below the upper limit, to be kept
      # track which data points have been removed and why
      which_remove <- which(df_step1[ , column_of_interest] > upper_limit)
    } else { # if (include_lower_limit == TRUE) # data of exactly the upper_limit will be kept
      which_in_range <- which(df_step1[ , column_of_interest] < upper_limit) # find the data below the upper_limit, to be kept
      # track which data points have been removed and why
      which_remove <- which(df_step1[ , column_of_interest] >= upper_limit)
    } # if (include_lower_limit == TRUE) # data of exactly the upper_limit will be kept
    df_step2 <- df_step1[which_in_range, ] # keep only data within the specified range
    # print(paste((dim(df_step1)[1] - dim(df_step2)[1])," data points were removed due to having ",column_of_interest," above ",upper_limit,sep = "")) # COMMENT
    remove_df_UL <- df_step1[which_remove, ]
    #if (dim(remove_df_UL)[1]>1) {
    if (dim(remove_df_UL)[1]>0) {  
      remove_df_UL$Reason <- reason_removed
    }
    rm(which_in_range,which_remove)
  } else { # if (is.na(upper_limit) == FALSE) { # an upper limit has been set
    df_step2 <- df_step1 # don't change the data
    remove_df_UL <- df_step1[0, ]
    remove_df_UL$Reason <- df_step1[0,1]
  } # if (is.na(upper_limit) == FALSE) { # an upper limit has been set
  
  # remove data below the lower limit
  if (is.na(lower_limit) == FALSE) { # an lower limit has been set
    if (include_lower_limit == TRUE) {# data of exactly the lower_limit will be kept
      which_in_range <- which(df_step2[ , column_of_interest] >= lower_limit) # find the data at or above the lower limit, to be kept
      which_remove <- which(df_step2[ , column_of_interest] < lower_limit) # track which data points have been removed and why
    } else { # if (is.na(include_lower_limit) == TRUE) # data of exactly the lower_limit will be kept
      which_in_range <- which(df_step2[ , column_of_interest] > lower_limit) # find the data above the lower_limit, to be kept
      which_remove <- which(df_step2[ , column_of_interest] <= lower_limit) # track which data points have been removed and why
    } # if (is.na(include_lower_limit) == TRUE) # data of exactly the lower_limit will be kept
    df_keep <- df_step2[which_in_range, ] # keep only data within the specified range
    # print(paste((dim(df_step2)[1] - dim(df_keep)[1])," data points were removed due to having ",column_of_interest," below ",lower_limit,sep = "")) # COMMENT
    remove_df_LL <- df_step2[which_remove, ]
    if (dim(remove_df_LL)[1]>1) {
      remove_df_LL$Reason <- reason_removed
    }
    rm(which_in_range,which_remove)
  } else { # if (is.na(lower_limit) == FALSE) { # an lower limit has been set
    df_keep <- df_step2 # don't change the data
    remove_df_LL <- df_step2[0, ]
    remove_df_LL$Reason <- df_step2[0,1]
  } # if (is.na(lower_limit) == FALSE) { # an lower limit has been set
  
  df_remove <- rbind(remove_df_NA, remove_df_UL, remove_df_LL)
  df_out <- list(df_keep,df_remove)
  if (dim(df_in)[1] != dim(df_keep)[1]+dim(df_remove)[1]) {
    stop("number of rows not adding up correctly")
  }
  return(df_out)
} # end of remove_data_outside_range.fn function

# remove data points outside specified range of values
remove_data_2_criteria.fn <- function(df_in, column_of_interest1, upper_limit1 = NA, remove_NAs1 = TRUE, column_of_interest2, upper_limit2 = NA, lower_limit2 = NA, remove_NAs2 = TRUE, reason_removed = " ") {
  
  # remove NA values (step1)
  if (remove_NAs1 == TRUE & remove_NAs2 == TRUE) { # NA values should be removed
    which_not_NA <- which(is.na(df_in[ , column_of_interest1]) == FALSE & is.na(df_in[ , column_of_interest2]) == FALSE) # find the not-NAs
    df_step1 <- df_in[which_not_NA, ] # keep all data that doesn't have NA in the column of interest
    # print(paste((dim(df_in)[1] - dim(df_step1)[1])," data points were removed due to having ",column_of_interest1," or ",column_of_interest2," as NA ",sep = "")) # COMMENT
    
    # track which data points have been removed and why
    which_remove <- which(is.na(df_in[ , column_of_interest2]) == TRUE | is.na(df_in[ , column_of_interest2]) == TRUE) # find the NAs
    remove_df_NA <- df_in[which_remove, ]
    if (dim(remove_df_NA)[1]>1) { # input reason for data removal into data frame
      remove_df_NA$Reason <- reason_removed
    } # if (dim(remove_df_NA)[1]>1) { # input reason for data removal
    rm(which_not_NA,which_remove)
  } else if (remove_NAs1 == TRUE & remove_NAs2 == FALSE) {
    stop("Finish writing code in remove_data_2_criteria.fn")
  } else if (remove_NAs1 == FALSE & remove_NAs2 == TRUE) {
    stop("Finish writing code in remove_data_2_criteria.fn")
  } else if (remove_NAs1 == FALSE & remove_NAs2 == FALSE) { # NA values should not be removed
    df_step1 <- df_in
    remove_df_NA <- df_in[0,]
  }
  
  # remove data above fitting the criteria (step2)
  if (class(upper_limit1) == "character" & class(upper_limit2) == "Date") {
    which_remove <- which(df_step1[ , column_of_interest1] == upper_limit1 & df_step1[ , column_of_interest2] >= lower_limit2 & df_step1[ , column_of_interest2] <= upper_limit2)
    #print(paste(length(which_remove)," data points were removed due to having ",column_of_interest1," of ",upper_limit1," and ",column_of_interest2," in the range ",lower_limit2,"-",upper_limit2,sep = "")) # COMMENT
    remove_df_criteria <- df_step1[which_remove, ]
    which_in_range <- which_rows_remain.fn(df_start = df_step1, which_rows_subset1 = which_remove) # figure out which data points remain
    df_keep <- df_step1[which_in_range, ] # keep only data within the specified range
    if (dim(remove_df_criteria)[1]>0) {
      remove_df_criteria$Reason <- reason_removed
    }
    rm(which_in_range,which_remove)
  } else {
    stop("finish writing code for remove_data_2_criteria.fn")
  }
  df_remove <- rbind(remove_df_NA, remove_df_criteria)
  df_out <- list(df_keep,df_remove)
  checksum.fn(N_original = dim(df_in)[1], part_A = dim(df_keep)[1], part_B = dim(df_remove)[1])
  return(df_out)
} # end of remove_data_outside_range.fn function

which_rows_remain.fn <- function(df_start,which_rows_subset1) {
  dummy_vec <- c(1:dim(df_start)[1])
  which_rows_subset2 <- which(dummy_vec %!in% which_rows_subset1)
  rm(dummy_vec)
  checksum.fn(N_original = dim(df_start)[1],part_A = length(which_rows_subset1),part_B = length(which_rows_subset2))
  return(which_rows_subset2)
} # end of which_rows_remain.fn function

# merge time-varying datasets
merge_time_varying_data.fn <- function(ML_input_in,predictor_data,latitude_col_s,longitude_col_s,datum_col_s,Dates_col_s,predictor_set_merged = NA, lag_n_days = 0, no_match_value = NA) {
  # no_match_value is the value to be input when there is no match, default is NA
  #### round lat/lon and recognize dates as dates for the two data frames to be joined, de-duplicate ####
  ML_input_in$Latitude <- round(ML_input_in$Latitude, 5)
  ML_input_in$Longitude <- round(ML_input_in$Longitude, 5)
  ML_input_in$Date <- as.Date(ML_input_in$Date,determine_date_format.fn(ML_input_in[1,"Date"]))#"%Y-%m-%d") # recognize dates as dates
  ML_input_in_no_repeats <- ML_input_in[!duplicated(ML_input_in), ] # remove any repeated rows
  predictor_data[ , latitude_col_s] <- round(predictor_data[ , latitude_col_s], 5)
  predictor_data[ , longitude_col_s] <- round(predictor_data[ , longitude_col_s], 5)
  predictor_data[ , Dates_col_s] <- as.Date(predictor_data[ , Dates_col_s],determine_date_format.fn(predictor_data[1, Dates_col_s]))#"%Y-%m-%d") # recognize dates as dates
  predictor_data_no_repeats <- predictor_data[!duplicated(predictor_data), ] # remove any repeated rows
  
  # join data sets using custom function:
  ML_input_out_list <- lapply(X = 1:dim(ML_input_in)[1],FUN = function(row_number){
    #print(names(predictor_data_no_repeats)) 
    #row_number <- x
    #print(row_number)
    ML_input_row <- ML_input_in[row_number, ]
    this_lat <- ML_input_row$Latitude
    N_dec_lat <- decimalplaces(this_lat) # how many decimal places are in the latitude variable?
    this_lon <- ML_input_row$Longitude
    N_dec_lon <- decimalplaces(this_lon) # how many decimal places are in the longitude variable?
    this_Date <- ML_input_row$Date
    
    # make sure we're matching on date or the correct lag date
    which_match_date <- which(predictor_data_no_repeats$Date == (this_Date-lag_n_days))
    length(which_match_date)
    predictor_data_date <- predictor_data_no_repeats[which_match_date, ]
    
    # match on latitude & longitude
    which_match <- which(round(predictor_data_date[ , c("Latitude")],N_dec_lat) == round(ML_input_row$Latitude,N_dec_lat) & 
                           round(predictor_data_date[ , c("Longitude")],N_dec_lon) == round(ML_input_row$Longitude,N_dec_lon))
    if (length(which_match)>0) { # is there a match?
      predictor_row_step1 <- predictor_data_date[which_match, ] # isolate matching data
      predictor_row_step1$Longitude <- round(predictor_row_step1$Longitude,N_dec_lon) # round longitudes
      predictor_row_step1$Latitude <- round(predictor_row_step1$Latitude,N_dec_lon) # round latitudes
      predictor_row_all_col <- predictor_row_step1[!duplicated(predictor_row_step1), ] # de-duplicate rows of data
      match_found <- 1
      if (dim(predictor_row_all_col)[1]>=1 & predictor_set_merged == "Fire_MODIS") { # multiple rows of data. Investigate and write more code  
        predictor_row_all_col_copy <- predictor_row_all_col
        predictor_row_all_col_new <- predictor_row_all_col[1, ] # take first row to get column names
        for (col_i in 1:dim(predictor_row_all_col_new)[2]) { # cycle through columns to take max value
          predictor_row_all_col_new[1, col_i] <- max(predictor_row_all_col_copy[ ,col_i])
        } # for (col_i in 1:dim(predictor_row_all_col_new)[2]) { # cycle through columns to take max value
        rm(predictor_row_all_col)
        predictor_row_all_col <- predictor_row_all_col_new
        rm(predictor_row_all_col_copy,predictor_row_all_col_new)
      }  else if (dim(predictor_row_all_col)[1]>=1 & predictor_set_merged %in% c("NAM_data","MAIAC_data","NDVI_data")) {  
        #print(predictor_set_merged) #COMMENT
        predictor_row_all_col_copy <- predictor_row_all_col
        predictor_row_all_col_new <- predictor_row_all_col[1, ] # take first row to get column names
        for (col_i in 1:dim(predictor_row_all_col_new)[2]) { # cycle through columns to take mean value
          #print(class(predictor_row_all_col_copy[ ,col_i])) # COMMENT
          if (class(predictor_row_all_col_copy[ ,col_i]) != "character") { # can't take average of a character
            predictor_row_all_col_new[1, col_i] <- mean(predictor_row_all_col_copy[ ,col_i])
          } else { # take unique value for characters
            predictor_row_all_col_new[1, col_i] <- as.character(unique(predictor_row_all_col_copy[ ,col_i]))  
          }
        } # for (col_i in 1:dim(predictor_row_all_col_new)[2]) { # cycle through columns to take max value
        
        rm(predictor_row_all_col)
        predictor_row_all_col <- predictor_row_all_col_new
        rm(predictor_row_all_col_copy,predictor_row_all_col_new)
      } else if  (dim(predictor_row_all_col)[1]>1) { 
        #stop(paste("multiple rows of data. Investigate and write more code. Data set: ",predictor_set_merged," x =",x,"Date =",this_Date)) # ,"Names of Variables:",names(predictor_row_step1)
        stop(paste("multiple rows of data. Investigate and write more code. Data set: ",predictor_set_merged," x =",row_number,"Date =",this_Date)) # ,"Names of Variables:",names(predictor_row_step1)
      } # multiple rows of data. Investigate and write more code
    } else { # if (length(which_match)>0) { # is there a match?
      predictor_row_all_col <- predictor_data_date[1, ] # get column names
      drop_cols <- c("Latitude","Longitude","Date") # define unnecessary columns
      keep_cols <- which(names(predictor_row_all_col) %!in% drop_cols)
      predictor_row_all_col[1,drop_cols] <- NA #no_match_value#NA
      predictor_row_all_col[1,keep_cols] <- no_match_value#NA
    } # if (length(which_match)>0) { # is there a match?
    
    # remove extraneous columns
    drop_cols <- c("Latitude","Longitude","Date") # define unnecessary columns
    keep_cols <- which(names(predictor_row_all_col) %!in% drop_cols)
    keep_names <- names(predictor_row_all_col[keep_cols])
    predictor_row <- data.frame(matrix(NA,nrow = 1,ncol = length(keep_names))) # create data frame
    names(predictor_row) <- keep_names
    predictor_row[1 , ] <- predictor_row_all_col[1, keep_cols]
    
    ML_input_out_row <- cbind(ML_input_row,predictor_row)
    if (dim(ML_input_out_row)[1]!=1) {stop(paste("ML_input_out_row has",dim(ML_input_out_row)[1],"rows of data, but it should have exactly 1 row.",predictor_set_merged,
                                                 this_Date,"row number",row_number))}
    return(ML_input_out_row)
  }) # end of ML_input_out_list lapply
  ML_input_out <- do.call("rbind",ML_input_out_list)
  return(ML_input_out)
} # end of merge_time_varying_data.fn function

# merge temporally static datasets
merge_time_static_data.fn <- function(ML_input_in,predictor_data,latitude_col_s,longitude_col_s, predictor_set_merged = NA) {
  # round lat/lon and recognize dates as dates for the two data frames to be joined
  ML_input_in$Latitude <- round(ML_input_in$Latitude, 5)
  ML_input_in$Longitude <- round(ML_input_in$Longitude, 5)
  predictor_data[ , latitude_col_s] <- round(predictor_data[ , latitude_col_s], 5)
  predictor_data[ , longitude_col_s] <- round(predictor_data[ , longitude_col_s], 5)
  
  # join data sets using the join function:
  #ML_input_out <- join(x = ML_input_in, y = predictor_data, by = c( "Latitude" = latitude_col_s, "Longitude" = longitude_col_s)) # join data sets
  
  # join data sets using custom function:
  ML_input_out_list <- lapply(X = 1:dim(ML_input_in)[1],FUN = function(row_number){
    ML_input_row <- ML_input_in[row_number, ]
    this_lat <- ML_input_row$Latitude
    N_dec_lat <- decimalplaces(this_lat) # how many decimal places are in the latitude variable?
    this_lon <- ML_input_row$Longitude
    N_dec_lon <- decimalplaces(this_lon) # how many decimal places are in the longitude variable?
    
    # match on latitude & longitude
    which_match <- which(round(predictor_data[ , c("Latitude")],N_dec_lat) == round(ML_input_row$Latitude,N_dec_lat) & 
                           round(predictor_data[ , c("Longitude")],N_dec_lon) == round(ML_input_row$Longitude,N_dec_lon))
    if (length(which_match)==0) { # no matches - try with fewer decimals
      if (N_dec_lat == 5) {
        new_N_dec_lat <- 4
      } else {
        new_N_dec_lat <- N_dec_lat
      }
      if (N_dec_lon == 5) {
        new_N_dec_lon <- 4
      } else {
        new_N_dec_lon <- N_dec_lon
      }
      which_match <- which(round(predictor_data[ , c("Latitude")],new_N_dec_lat) == round(ML_input_row$Latitude,new_N_dec_lat) & 
                             round(predictor_data[ , c("Longitude")],new_N_dec_lon) == round(ML_input_row$Longitude,new_N_dec_lon))
    }
    if (length(which_match)>0) { # is there a match?
      predictor_row_step1 <- predictor_data[which_match, ] # isolate matching data
      predictor_row_step1$Longitude <- round(predictor_row_step1$Longitude,N_dec_lon) # round longitudes
      predictor_row_step1$Latitude <- round(predictor_row_step1$Latitude,N_dec_lat) # round latitudes
      predictor_row_all_col_step <- predictor_row_step1[!duplicated(predictor_row_step1), ] # de-duplicate rows of data
      match_found <- 1
      if (dim(predictor_row_all_col_step)[1] == 1) {
        predictor_row_all_col <- predictor_row_all_col_step
        #} else if (dim(predictor_row_all_col_step)[1] > 1 & predictor_set_merged == "Highways") {
      } else if (dim(predictor_row_all_col_step)[1] > 1 & predictor_set_merged %in% c("Highways","NED_data", "NLCD_data", "Population")) {  
        predictor_row_all_col <- predictor_row_all_col_step[1, ]# take first row to get column names, etc
        for (this_col in 1:dim(predictor_row_all_col)[2]) { # take average value of each column
         predictor_row_all_col[1,this_col] <- mean(as.numeric(as.character(predictor_row_all_col_step[ , this_col])))
        } # for (this_col in 1:dim(predictor_row_all_col)[2]) { # take average value of each column
      } else if (dim(predictor_row_all_col_step)[1] > 1 & predictor_set_merged != "Highways") {
        stop(paste("multiple rows of data. Investigate and write more code. x = ",row_number,"Data set:",predictor_set_merged))# row_number
      } # multiple rows of data. Investigate and write more code
    } else { # if (length(which_match)>0) { # is there a match?
      #print(paste("****no match found. Data set:",predictor_set_merged,"****")) # COMMENT
      match_found <- 0
    } # if (length(which_match)>0) { # is there a match?
    # remove extraneous columns
    drop_cols <- c("Latitude","Longitude") # define unnecessary columns
    keep_cols <- which(names(predictor_data) %!in% drop_cols)
    keep_names <- names(predictor_data[keep_cols])
    predictor_row <- data.frame(matrix(NA,nrow = 1,ncol = length(keep_names))) # create data frame
    names(predictor_row) <- keep_names
    
    if (match_found == 1) { # match was found
      # # remove extraneous columns
      # drop_cols <- c("Latitude","Longitude") # define unnecessary columns
      # keep_cols <- which(names(predictor_row_all_col) %!in% drop_cols)
      # keep_names <- names(predictor_row_all_col[keep_cols])
      # predictor_row <- data.frame(matrix(NA,nrow = 1,ncol = length(keep_names))) # create data frame
      # names(predictor_row) <- keep_names
      predictor_row[1 , ] <- predictor_row_all_col[1, keep_cols]
    } else { # if (match_found == 1) { # match was found
      #print(paste("***write more code to accomodate no match being found in merge_time_varying_data.fn function. Data set:",predictor_set_merged,"***"))
      # print(paste("***no match found in merge_time_varying_data.fn function. Data set:",predictor_set_merged,"***")) # COMMENT
    } # if (match_found == 1) { # match was found
    
    ML_input_out_row <- cbind(ML_input_row,predictor_row)
    return(ML_input_out_row)
  }) # end of ML_input_out_list lapply
  ML_input_out <- do.call("rbind",ML_input_out_list)
  return(ML_input_out)
} # end of merge_time_static_data.fn function

# replace column names for data frame
replace_column_names.fn <- function(df_in,old_col_name,new_col_name) {
  # df_in <- Highways_data
  # old_col_name <- "Datum" #"Lat"
  # new_col_name <- "NewName" #"Latitude"
  
  if (old_col_name %in% colnames(df_in)) { # check if the old_col_name is a column name in df_in
    #Highways_data <- Highways_data[ , !(names(Highways_data) %in% )]
    which_col <- which(colnames(df_in) == old_col_name)
    colnames(df_in)[which_col] <- new_col_name
  } # if (old_col_name %in% colnames(df_in)) { # check if the old_col_name is a column name in df_in
  
  return(df_in) # output from function
} # end of replace_column_names.fn function

# Load and Merge Fire Modis Data
merge_Fire_MODIS_data.fn <- function(Buffer_radius_km, ML_input, Fire_MODIS_file_name,ProcessedData.directory,predictor_sub_folder,this_Date, lag_n_days = 0, no_match_value) {#, study_start_date, study_stop_date) {
  #### Define dates and columns ####
  this_lag_Date <- this_Date-lag_n_days
  rm(this_Date)
  latitude_col_s <- "Latitude" # define latitude column
  longitude_col_s <- "Longitude" # define longitude column
  Dates_col_s <- "Date" # define date column
  new_col_name <- paste("Fire_Count_",Buffer_radius_km,"km_lag",lag_n_days,"days",sep = "")
  #### Load data for this date from all of the input files for this variable ####
  Fire_MODIS_data_list <- lapply(X = 1:length(Fire_MODIS_file_name), FUN = function(file_i) {#list() # create list for merging all data sets
    print(paste("Processing Fire_MODIS file ",Fire_MODIS_file_name[file_i],sep = "")) # COMMENT
    name_no_suffix <- substr(Fire_MODIS_file_name[file_i],1,nchar(Fire_MODIS_file_name[file_i])-4)
    Fire_MODIS_file_nameDate <- file.path(define_file_paths.fn("ProcessedData.directory"),predictor_sub_folder,name_no_suffix,paste(name_no_suffix,'_',this_lag_Date,'.csv',sep = ""))
    if (file.exists(Fire_MODIS_file_nameDate)) { # check if a file exists for this date
      Fire_MODIS_data_step <- read.csv(file.path(Fire_MODIS_file_nameDate),header=TRUE) # load data file
      #Fire_MODIS_data_step <- read.csv(file.path(ProcessedData.directory,predictor_sub_folder, Fire_MODIS_file_nameDate),header=TRUE) # load data file
      #Fire_MODIS_data_step <- read.csv(file.path(ProcessedData.directory,predictor_sub_folder, Fire_MODIS_file_name[file_i]),header=TRUE) # load data file
      Fire_MODIS_data_step<- as.data.frame(Fire_MODIS_data_step) # define data as data frame
      this_date_format <- determine_date_format.fn(Fire_MODIS_data_step[1,Dates_col_s])
      Fire_MODIS_data_step[ , c(Dates_col_s)] <- as.Date(Fire_MODIS_data_step[ , c(Dates_col_s)],this_date_format) # recognize dates as dates
      # change column names
        Fire_MODIS_data_step <- replace_column_names.fn(df_in = Fire_MODIS_data_step, old_col_name = "Lat", new_col_name = "Latitude") # replace "Lat" with "Latitude"
        Fire_MODIS_data_step <- replace_column_names.fn(df_in = Fire_MODIS_data_step, old_col_name = "Lon", new_col_name = "Longitude") # replace "Lat" with "Latitude"
        Fire_MODIS_data_step <- replace_column_names.fn(df_in = Fire_MODIS_data_step, old_col_name = "Fire_Count", new_col_name = new_col_name) # replace column name
        Fire_MODIS_data_step <- replace_column_names.fn(df_in = Fire_MODIS_data_step, old_col_name = "fire_count", new_col_name = new_col_name) # replace column name
      # check on data
        Check_data <- check_4_NAs.fn(no_NAs_allowed_cols = c("Latitude","Longitude","Date"), input_data = Fire_MODIS_data_step)
        if (length(Check_data)>0) {stop("***Check_4_NAs.fn found questionable data. Investigate.***")}
        rm(Check_data)
      # remove extraneous columns
        drop_cols <- c("Datum","Easting","Northing","old_lon","old_lat","old_Datum","X") # define unnecessary columns
        Fire_MODIS_data_step <- Fire_MODIS_data_step[ , !(names(Fire_MODIS_data_step) %in% drop_cols)] # drop unnecessary columns
      # isolate data for this date
        which_this_date <- which(Fire_MODIS_data_step[ , c(Dates_col_s)] == this_lag_Date) # which rows in the Fire_MODIS data are for this date?
        if (length(which_this_date) > 0) { # is there data for this date in this file?
          #print(paste("There is Fire_MODIS data for lag date",this_lag_Date," in ",Fire_MODIS_file_name[file_i])) #  # COMMENT
          Fire_MODIS_data_date_step <- Fire_MODIS_data_step[which_this_date, ] # isolate data for this date
          rm(Fire_MODIS_data_step) # clear variable
          Fire_MODIS_data_date <- Fire_MODIS_data_date_step[!duplicated(Fire_MODIS_data_date_step), ] # de-duplicate rows of data
          rm(Fire_MODIS_data_date_step)
          #Fire_MODIS_data_list[[Fire_MODIS_file_name[file_i]]] <- Fire_MODIS_data_date # input data into list
          #rm(Fire_MODIS_data_date)
        } else { # if (length(which_this_date) > 0) { # is there data for this date in this file? - No
          #print(paste("No Fire_MODIS data for lag date =",this_lag_Date,"in",Fire_MODIS_file_name[file_i])) # COMMENT
          Fire_MODIS_data_date <- Fire_MODIS_data_step[1 , ] # grab first row to get column number/names
          rm(Fire_MODIS_data_step) # clear variable
          Fire_MODIS_data_date[1, ] <- NA # replace data with NA
        } # if (length(which_this_date) > 0) { # is there data for this date in this file?
    } else { # if (file.exists(Fire_MODIS_file_nameDate)) { # check if a file exists for this date - No
      print(paste("There is no Fire MODIS data for",this_lag_Date))
      # # the date in question did not have a file, so the file from Jan 1, 2008 is being read in to get the column names
      # Fire_MODIS_file_nameJan1_2008 <- file.path(define_file_paths.fn("ProcessedData.directory"),predictor_sub_folder,name_no_suffix,paste(name_no_suffix,'_',"2008-01-01",'.csv',sep = ""))
      # Fire_MODIS_data_step <- read.csv(file.path(Fire_MODIS_file_nameJan1_2008),header=TRUE) # load data file
      # the date in question did not have a file, so the file from Jan 1, 2008 is being read in to get the column names
      Fire_MODIS_file_nameJan2_2008 <- file.path(define_file_paths.fn("ProcessedData.directory"),predictor_sub_folder,name_no_suffix,paste(name_no_suffix,'_',"2008-01-02",'.csv',sep = ""))
      Fire_MODIS_data_step <- read.csv(file.path(Fire_MODIS_file_nameJan2_2008),header=TRUE) # load data file
      
      #Fire_MODIS_data_step <- read.csv(file.path(ProcessedData.directory,predictor_sub_folder, Fire_MODIS_file_nameJan1_2008),header=TRUE) # load data file
      Fire_MODIS_data_step<- as.data.frame(Fire_MODIS_data_step) # define data as data frame
      this_date_format <- determine_date_format.fn(Fire_MODIS_data_step[1,Dates_col_s])
      Fire_MODIS_data_step[ , c(Dates_col_s)] <- as.Date(Fire_MODIS_data_step[ , c(Dates_col_s)],this_date_format) # recognize dates as dates
      # change column names
      Fire_MODIS_data_step <- replace_column_names.fn(df_in = Fire_MODIS_data_step, old_col_name = "Lat", new_col_name = "Latitude") # replace "Lat" with "Latitude"
      Fire_MODIS_data_step <- replace_column_names.fn(df_in = Fire_MODIS_data_step, old_col_name = "Lon", new_col_name = "Longitude") # replace "Lat" with "Latitude"
      Fire_MODIS_data_step <- replace_column_names.fn(df_in = Fire_MODIS_data_step, old_col_name = "Fire_Count", new_col_name = new_col_name) # replace column name
      Fire_MODIS_data_step <- replace_column_names.fn(df_in = Fire_MODIS_data_step, old_col_name = "fire_count", new_col_name = new_col_name) # replace column name
      # check on data
      Check_data <- check_4_NAs.fn(no_NAs_allowed_cols = c("Latitude","Longitude","Date"), input_data = Fire_MODIS_data_step)
      if (length(Check_data)>0) {stop("***Check_4_NAs.fn found questionable data. Investigate.***")}
      rm(Check_data)
      # remove extraneous columns
      drop_cols <- c("Datum","Easting","Northing","old_lon","old_lat","old_Datum","X") # define unnecessary columns
      Fire_MODIS_data_step <- Fire_MODIS_data_step[ , !(names(Fire_MODIS_data_step) %in% drop_cols)] # drop unnecessary columns
      Fire_MODIS_data_date <- Fire_MODIS_data_step[1 , ] # grab first row to get column number/names
      rm(Fire_MODIS_data_step) # clear variable
      Fire_MODIS_data_date[1, ] <- NA # replace data with NA
    } # if (file.exists(Fire_MODIS_file_nameDate)) { # check if a file exists for this date
      #rm(Fire_MODIS_data_step) # clear variable
    return(Fire_MODIS_data_date)
  }) # end of Fire_MODIS_data_list <- lapply(X = 1:length(Fire_MODIS_file_name), FUN = function(file_i) {#list() # create list for merging all data sets 
  Fire_MODIS_data_w_dups <- do.call("rbind", Fire_MODIS_data_list) # unlist data from various files
  Fire_MODIS_data <- Fire_MODIS_data_w_dups[!duplicated(Fire_MODIS_data_w_dups),] # de-duplicate rows of data
  rm(Fire_MODIS_data_w_dups,Fire_MODIS_data_list)
  #### Merge Fire_MODIS data into ML_inputs
  if (!is.null(Fire_MODIS_data)) { # merge No Fire_MODIS data if there is any for this date
    ML_input <- merge_time_varying_data.fn(ML_input_in = ML_input, predictor_data = Fire_MODIS_data,latitude_col_s = latitude_col_s,longitude_col_s = longitude_col_s, datum_col_s = datum_col_s,Dates_col_s = Dates_col_s,predictor_set_merged = "Fire_MODIS", lag_n_days = lag_n_days, no_match_value) # join wrapper function
  } # if (!is.null(Fire_MODIS_data)) { # merge No Fire_MODIS data if there is any for this date
  rm(Fire_MODIS_data) # clear variables
  # add column as space holder if there was no data
  if (new_col_name %!in% colnames(ML_input)) { # add column as space holder if there was no data
    ML_input[, new_col_name] <- no_match_value #NA # add column as space holder if there was no data
  } # add column as space holder if there was no data
  return(ML_input) # output from function
} # end of merge_Fire_MODIS_data.fn function

# Load and merge GASP Data
merge_GASP_data.fn <- function(ML_input, GASP_file_name,ProcessedData.directory,predictor_sub_folder, this_Date) { # study_start_date, study_stop_date) {
  GASP_data_list <- list() # create list for merging all data sets
  latitude_col_s <- "Latitude"
  longitude_col_s <- "Longitude"
  Dates_col_s <- "Date"
  for (file_i in 1:length(GASP_file_name)) { # Load and merge all GASP Data files
    #print(paste("Processing GASP file",GASP_file_name[file_i])) # COMMENT
    GASP_data_step <- read.csv(file.path(ProcessedData.directory,predictor_sub_folder, GASP_file_name[file_i]),header=TRUE) # load the AQS file
    GASP_data_step<- as.data.frame(GASP_data_step)
    GASP_data_step[ , c(Dates_col_s)] <- as.Date(GASP_data_step[ , c(Dates_col_s)],"%Y-%m-%d") # recognize dates as dates
    
    # change column names
    GASP_data_step <- replace_column_names.fn(df_in = GASP_data_step, old_col_name = "Lat", new_col_name = "Latitude") # replace "Lat" with "Latitude"
    GASP_data_step <- replace_column_names.fn(df_in = GASP_data_step, old_col_name = "Lon", new_col_name = "Longitude") # replace "Lat" with "Latitude"
    GASP_data_step <- replace_column_names.fn(df_in = GASP_data_step, old_col_name = "AOD", new_col_name = "GASP_AOD") # replace "Lat" with "Latitude"
    
    # remove extraneous columns
    drop_cols <- c("Datum","Easting","Northing","old_lon","old_lat","old_Datum","X","old_datum") # define unnecessary columns
    GASP_data_step <- GASP_data_step[ , !(names(GASP_data_step) %in% drop_cols)] # drop unnecessary columns
    # remove extraneous columns and dates that are not this_Date
    #GASP_data <- GASP_data_step[which_this_date,c("Latitude","Longitude","Date","GASP_AOD")]
    
    Check_data <- check_4_NAs.fn(no_NAs_allowed_cols = c("Latitude","Longitude","Date"), input_data = GASP_data_step)
    if (length(Check_data)>0) {stop("***Check_4_NAs.fn found questionable data. Investigate.***")}
    rm(Check_data)
    
    # isolate data for this date
    which_this_date <- which(GASP_data_step[ , c(Dates_col_s)] == this_Date)
    if (length(which_this_date) > 0) { # is there data for this date?
      #print(paste("There is GASP data for ",this_Date," in ",GASP_file_name[file_i])) # COMMENT
      GASP_data_date <- GASP_data_step[which_this_date, ] # isolate data for this date
      GASP_data_list[[GASP_file_name[file_i]]] <- GASP_data_date # input data into list
      rm(GASP_data_date)
    } else { # if (length(which_this_date) > 0) { # is there data for this date? - No
      #print(paste("No GASP data for ",this_Date," in ",GASP_file_name[file_i])) # COMMENT
    } # if (length(which_this_date) > 0) { # is there data for this date?
    rm(GASP_data_step) # clear variable
  } # for (file_i in 1:length(GASP_file_name)) { # Load and merge all GASP Data files
  GASP_data_w_dups <- do.call("rbind", GASP_data_list) # unlist data from various files
  GASP_data <- GASP_data_w_dups[!duplicated(GASP_data_w_dups),] # de-duplicate rows of data
  rm(GASP_data_w_dups,GASP_data_list)
  if (!is.null(GASP_data)) { # merge GASP data if there is any for this date
    ML_input <- merge_time_varying_data.fn(ML_input_in = ML_input, predictor_data = GASP_data, latitude_col_s = latitude_col_s, longitude_col_s = longitude_col_s, datum_col_s = datum_col_s,Dates_col_s = Dates_col_s, predictor_set_merged = "GASP_data") # join wrapper function
  } # if (!is.null(GASP_data)) { # merge GASP data if there is any for this date
  rm(GASP_data)
  # add column as space holder if there was no data
  if ("GASP_AOD" %!in% colnames(ML_input)) { # add column as space holder if there was no data
    ML_input$GASP_AOD <- NA # add column as space holder if there was no data
  } # add column as space holder if there was no data
  return(ML_input)
} # end of merge_GASP_data.fn function

# Load and merge Highways Data
merge_Highways_data.fn <- function(ML_input, Highways_file_name, ProcessedData.directory,predictor_sub_folder) { # study_start_date, study_stop_date) {  # Load and merge Highways Data
  Highways_data_list <- list()
  latitude_col_s <- "Latitude"
  longitude_col_s <- "Longitude"
  #datum_col_s <- "Datum"
  for (file_i in 1:length(Highways_file_name)) { # Load and merge all Highways Data files
    print(paste("Processing ",Highways_file_name[file_i])) # COMMENT
    Highways_data_step <- read.csv(file.path(ProcessedData.directory,predictor_sub_folder, Highways_file_name[file_i]),header=TRUE) # load the file
    Highways_data_step<- as.data.frame(Highways_data_step)
    
    # change column names
    Highways_data_step <- replace_column_names.fn(df_in = Highways_data_step, old_col_name = "Lat", new_col_name = "Latitude") # replace "Lat" with "Latitude"
    Highways_data_step <- replace_column_names.fn(df_in = Highways_data_step, old_col_name = "Lon", new_col_name = "Longitude") # replace "Lat" with "Latitude"
    
    # remove extraneous columns
    drop_cols <- c("X","Datum","old_lon","old_lat","old_Datum","Easting","Northing","State_FIPS","County_FIPS","Tract_code","ZCTA5_code")
    Highways_data_step <- Highways_data_step[ , !(names(Highways_data_step) %in% drop_cols)]
    print(names(Highways_data_step))
    
    Highways_data_list[[Highways_file_name[file_i]]] <- Highways_data_step
    rm(Highways_data_step)
  } # for (file_i in 1:length(Highways_file_name)) { # Load and merge all Highways Data files
  Highways_data_w_dups <- do.call("rbind", Highways_data_list) # unlist data from various files
  Highways_data <- Highways_data_w_dups[!duplicated(Highways_data_w_dups),] # de-duplicate rows of data
  
  Check_data <- check_4_NAs.fn(no_NAs_allowed_cols = c("Latitude","Longitude"), input_data = Highways_data)
  if (length(Check_data)>0) {stop("***Check_4_NAs.fn found questionable data. Investigate.***")}
  rm(Check_data)
  
  # join wrapper function
  ML_input <- merge_time_static_data.fn(ML_input_in = ML_input, predictor_data = Highways_data,latitude_col_s = latitude_col_s,longitude_col_s = longitude_col_s, predictor_set_merged = "Highways") 
  rm(Highways_data, Highways_data_list,Highways_data_w_dups) # clear variable
  return(ML_input)
} # end of merge_Highways_data.fn function

# Load and merge Population Data
merge_population_data.fn <- function(ML_input = ML_input, Pop_density_file_name = Pop_density_file_name, ProcessedData.directory = define_file_paths.fn("ProcessedData.directory"),predictor_sub_folder = predictor_sub_folder) { # load and merge population data
  Population_data_list <- list()
  latitude_col_s <- "Latitude"
  longitude_col_s <- "Longitude"
  for (file_i in 1:length(Pop_density_file_name)) { # Load and merge all Population Data files
    print(paste("Processing ",Pop_density_file_name[file_i])) # COMMENT
    Population_data_step <- read.csv(file.path(ProcessedData.directory,predictor_sub_folder, Pop_density_file_name[file_i]),header=TRUE) # load the file
    Population_data_step<- as.data.frame(Population_data_step)
    
    # change column names
    Population_data_step <- replace_column_names.fn(df_in = Population_data_step, old_col_name = "Lat", new_col_name = "Latitude") # replace "Lat" with "Latitude"
    Population_data_step <- replace_column_names.fn(df_in = Population_data_step, old_col_name = "Lon", new_col_name = "Longitude") # replace "Lat" with "Latitude"
    
    # remove extraneous columns
    drop_cols <- c("X","Datum","old_lon","old_lat","old_Datum","Easting","Northing","State_FIPS","County_FIPS","Tract_code","ZCTA5_code")
    Population_data_step <- Population_data_step[ , !(names(Population_data_step) %in% drop_cols)]
    print(names(Population_data_step))
    
    Population_data_list[[Pop_density_file_name[file_i]]] <- Population_data_step
    rm(Population_data_step)
  } # for (file_i in 1:length(Pop_density_file_name)) { # Load and merge all Population Data files
  Population_data_w_dups <- do.call("rbind", Population_data_list) # unlist data from various files
  Population_data <- Population_data_w_dups[!duplicated(Population_data_w_dups),] # de-duplicate rows of data
  
  Check_data <- check_4_NAs.fn(no_NAs_allowed_cols = c("Latitude","Longitude"), input_data = Population_data)
  if (length(Check_data)>0) {stop("***Check_4_NAs.fn found questionable data. Investigate.***")}
  rm(Check_data)
  
  # join wrapper function
  ML_input <- merge_time_static_data.fn(ML_input_in = ML_input, predictor_data = Population_data,latitude_col_s = latitude_col_s,longitude_col_s = longitude_col_s, predictor_set_merged = "Population") 
  rm(Population_data, Population_data_list,Population_data_w_dups) # clear variable
  return(ML_input)
} # end of merge_Population_data.fn function

# Load and merge MAIAC Data
merge_MAIAC_data.fn <- function(ML_input,MAIAC_file_name,ProcessedData.directory,predictor_sub_folder, this_Date) { #study_start_date, study_stop_date) {
  MAIAC_data_list <- list() # create list for merging all data sets
  latitude_col_s <- "Latitude"
  longitude_col_s <- "Longitude"
  Dates_col_s <- "Date"
  for (file_i in 1:length(MAIAC_file_name)) { # Load and merge all MAIAC Data files
    print(paste("Processing",MAIAC_file_name[file_i])) # COMMENT
    MAIAC_data_step <- read.csv(file.path(ProcessedData.directory,predictor_sub_folder, MAIAC_file_name[file_i]),header=TRUE, stringsAsFactors=FALSE) # load the AQS file
    MAIAC_data_step <- as.data.frame(MAIAC_data_step)
    date_format <- determine_date_format.fn(check_date = MAIAC_data_step[10, c(Dates_col_s)]) # determine format used for this MAIAC file; pick the 10th row as representative of the file (sometimes the first few rows are repeated headers)
    #print(date_format) # REMOVE
    MAIAC_data_step[ , c(Dates_col_s)] <- as.Date(MAIAC_data_step[ , c(Dates_col_s)],date_format) # recognize dates as dates
    options(warn  =  1) # dont' throw an error when there's a warning and stop the code from running further
    MAIAC_data_step$MAIAC_AOD <- as.numeric(MAIAC_data_step$MAIAC_AOD)
    options(warn  =  2) # throw an error when there's a warning and stop the code from running further
    #print(paste("Earliest date in file:",min(MAIAC_data_step[ , c(Dates_col_s)],na.rm = TRUE))) # COMMENT
    #plot.new()
    # #points(MAIAC_data_step$Date, MAIAC_data_step$MAIAC_AOD)
    #plot(MAIAC_data_step$Date, MAIAC_data_step$MAIAC_AOD)
    # change column names
    MAIAC_data_step <- replace_column_names.fn(df_in = MAIAC_data_step, old_col_name = "Lat", new_col_name = "Latitude") # replace "Lat" with "Latitude"
    MAIAC_data_step <- replace_column_names.fn(df_in = MAIAC_data_step, old_col_name = "Lon", new_col_name = "Longitude") # replace "Lat" with "Latitude"
    
    # remove extraneous columns
    drop_cols <- c("Datum","Easting","Northing","old_lon","old_lat","old_Datum","X") # define unnecessary columns
    MAIAC_data_step <- MAIAC_data_step[ , !(names(MAIAC_data_step) %in% drop_cols)] # drop unnecessary columns
    
    # isolate data for this date
    which_this_date <- which(MAIAC_data_step[ , c(Dates_col_s)] == this_Date)
    if (length(which_this_date) > 0) { # is there any data for this date?
      #print(paste("There is MAIAC data for ",this_Date," in ",MAIAC_file_name[file_i],sep = "")) # COMMENT
      MAIAC_data_date <- MAIAC_data_step[which_this_date, ] # isolate data for this date
      MAIAC_data_date[ , c(latitude_col_s)] <- as.numeric(as.character(MAIAC_data_date[ , c(latitude_col_s)])) # recognize latitude as numerical
      MAIAC_data_date[ , c(longitude_col_s)] <- as.numeric(as.character(MAIAC_data_date[ , c(longitude_col_s)])) # recognize longitude as numerical
      Check_data <- check_4_NAs.fn(no_NAs_allowed_cols = c("Latitude","Longitude","Date"), input_data = MAIAC_data_date)
      if (length(Check_data)>0) {print("***Check_4_NAs.fn found questionable data. Investigate.***")}
      rm(Check_data)
      Check_data <- check_4_NAs.fn(no_NAs_allowed_cols = c("Latitude","Longitude","Date"), input_data = MAIAC_data_date)
      if (length(Check_data)>0) {print("***Check_4_NAs.fn found questionable data. Investigate.***")}
      rm(Check_data)
      MAIAC_data_date_no_repeats <- MAIAC_data_date[!duplicated(MAIAC_data_date), ]
      MAIAC_data_list[[MAIAC_file_name[file_i]]] <- MAIAC_data_date_no_repeats # input data into list
      rm(MAIAC_data_date,MAIAC_data_date_no_repeats)
    } else { # if (length(which_this_date) > 0) { # is there any data for this date? - No
      #print(paste("No MAIAC data for ",this_Date," in ",MAIAC_file_name[file_i])) # COMMENT
      #  MAIAC_data_date <- MAIAC_data_step[20, ] # just grab 20th row as a place holder since nothing matches (preserve number of columns)
    } # if (length(which_this_date) > 0) { # is there any data for this date?  
    rm(MAIAC_data_step) # clear variable
  } # for (file_i in 1:length(MAIAC_file_name)) { # Load and merge all MAIAC Data files
  MAIAC_data_w_dups <- do.call("rbind", MAIAC_data_list) # unlist data from various files
  MAIAC_data <- MAIAC_data_w_dups[!duplicated(MAIAC_data_w_dups),] # de-duplicate rows of data
  
  if (!is.null(MAIAC_data)) { # merge MAIAC data if there is any for this date
    Check_data <- check_4_NAs.fn(no_NAs_allowed_cols = c("Latitude","Longitude","Date"), input_data = MAIAC_data)
    if (length(Check_data)>0) {stop("***Check_4_NAs.fn found questionable data. Investigate.***")}
    rm(Check_data)
    if (class(MAIAC_data$Date) != "Date") {stop("***class of Date_Local is not 'Date'. Investigate***")}
    if (class(MAIAC_data$Latitude) != "numeric") {stop("***class of Date_Local is not 'Date'. Investigate***")}
    rm(MAIAC_data_w_dups)
    ML_input <- merge_time_varying_data.fn(ML_input_in = ML_input, predictor_data = MAIAC_data,latitude_col_s = latitude_col_s,longitude_col_s = longitude_col_s, datum_col_s = datum_col_s,Dates_col_s = Dates_col_s,predictor_set_merged = "MAIAC_data") # join wrapper function
  } # if (!is.null(MAIAC_data)) { # merge GASP data if there is any for this date
  rm(MAIAC_data)
  
  # add column as space holder if there was no data
  if ("MAIAC_AOD" %!in% colnames(ML_input)) { # add column as space holder if there was no data
    ML_input$MAIAC_AOD <- NA # add column as space holder if there was no data
  } # add column as space holder if there was no data
  return(ML_input)
} # end of merge_MAIAC_data.fn function

# Load and merge NAM Data
merge_NAM_data.fn <- function(ML_input, NAM_file_name,task_counter,ProcessedData.directory,predictor_sub_folder,this_Date) {#, study_start_date, study_stop_date) {
  #NAM_data_list <- list() # create list for merging all data sets
  latitude_col_s <- "Latitude" # define latitude column
  longitude_col_s <- "Longitude" # define longitude column
  Dates_col_s <- "Date" # define date column
    NAM_file_name1 <- file.path(ProcessedData.directory,NAM_folder,NAM_sub_folder,NAM_sub_sub_folder,paste("NAM_Step5_",this_Date,"_batch",define_study_constants.fn("NAM_batch_date"),".csv",sep = "")) # get the NAM file for this_date
    NAM_file_name2 <- file.path(ProcessedData.directory,NAM_folder,NAM_sub_folder,NAM_sub_sub_folder,paste("NAM_Step5_",this_Date+1,"_batch",define_study_constants.fn("NAM_batch_date"),".csv",sep = "")) # get the NAM file for this_date+1 (UTC) so that all the hours for the local date will be included
    print(paste("Processing NAM files",NAM_file_name1,"and",NAM_file_name2))
    if (file.exists(NAM_file_name1) & file.exists(NAM_file_name2)) { # read in files if they exists
      NAM_data_step1 <- read.csv(file.path(NAM_file_name1),header=TRUE, stringsAsFactors = FALSE)  # load data file
      NAM_data_step2 <- read.csv(file.path(NAM_file_name2),header=TRUE, stringsAsFactors = FALSE)  # load data file
      NAM_data_step1<- as.data.frame(NAM_data_step1) # define data as data frame
      NAM_data_step2<- as.data.frame(NAM_data_step2) # define data as data frame
      NAM_data_step <- rbind(NAM_data_step1,NAM_data_step2) # rbind the files for the 2 UTC dates that will have data for the local date
      rm(NAM_data_step1,NAM_data_step2) # clear variables
    } else if (file.exists(NAM_file_name1) & !file.exists(NAM_file_name2)) { # only NAM_file_name1 exists
      NAM_data_step1 <- read.csv(file.path(NAM_file_name1),header=TRUE, stringsAsFactors = FALSE)  # load data file
      NAM_data_step<- as.data.frame(NAM_data_step1) # define data as data frame
      rm(NAM_data_step1) # clear variables
    } else if (!file.exists(NAM_file_name1) & file.exists(NAM_file_name2)) { # only NAM_file_name1 exists
      NAM_data_step2 <- read.csv(file.path(NAM_file_name2),header=TRUE, stringsAsFactors = FALSE)  # load data file
      NAM_data_step<- as.data.frame(NAM_data_step2) # define data as data frame
      rm(NAM_data_step2) # clear variables
   } # if (file.exists(NAM_file_name1) & file.exists(NAM_file_name2)) { # read in files if they exists
    rm(NAM_file_name1,NAM_file_name2) # clear variables
    
  if (exists("NAM_data_step")) { # process if any NAM data exists for this date
    # change column names
      NAM_data_step <- replace_column_names.fn(df_in = NAM_data_step, old_col_name = "Lat", new_col_name = "Latitude") # replace "Lat" with "Latitude"
      NAM_data_step <- replace_column_names.fn(df_in = NAM_data_step, old_col_name = "Lon", new_col_name = "Longitude") # replace "Lat" with "Latitude"
      NAM_data_step <- replace_column_names.fn(df_in = NAM_data_step, old_col_name = "Local.Date", new_col_name = "Date") # replace "Lat" with "Latitude"
    # recognize dates as dates
      NAM_data_step[ , c(Dates_col_s)] <- as.Date(NAM_data_step[ , c(Dates_col_s)],"%Y-%m-%d") 
    # remove extraneous columns
      drop_cols <- c("Datum","Easting","Northing","old_lon","old_lat","old_Datum","State_FIPS","County_FIPS","Tract_code","ZCTA5_code") # define unnecessary columns
      NAM_data_step <- NAM_data_step[ , !(names(NAM_data_step) %in% drop_cols)] # drop unnecessary columns
    # track column names
      NAM_colnames_step <- colnames(NAM_data_step)
      NAM_colnames <- NAM_colnames_step[NAM_colnames_step %!in% c("Latitude","Longitude","Date")]
      rm(NAM_colnames_step)
    # Check on data
      Check_data <- check_4_NAs.fn(no_NAs_allowed_cols = c("Latitude","Longitude","Date"), input_data = NAM_data_step)
      if (length(Check_data)>0) {stop(paste("***Check_4_NAs.fn found questionable NAM data. Investigate.***X =",X,"Date = ",this_Date))}
      rm(Check_data)
    # isolate data for this date
    which_this_date <- which(NAM_data_step[ , c(Dates_col_s)] == this_Date) # which rows in the NAM data are for this date?
      if (length(which_this_date) > 0) { # is there data for this date in this file?
        # print(paste("There is NAM data for ",this_Date," in ",NAM_file_name[file_i])) # COMMENT
        NAM_data_date <- NAM_data_step[which_this_date, ] # isolate data for this date
        rm(NAM_data_step) # clear variable
        NAM_data <- NAM_data_date[!duplicated(NAM_data_date), ] # de-duplicate rows of data
        rm(NAM_data_date)
      } else { # if (length(which_this_date) > 0) { # is there data for this date in this file? - No
        print(paste("No NAM data for",this_Date))#,"in",NAM_file_name[file_i])) # COMMENT
      } # if (length(which_this_date) > 0) { # is there data for this date in this file?
      
  }
  if (exists("NAM_data")) { # merge NAM data if there is any for this date
    ML_input <- merge_time_varying_data.fn(ML_input_in = ML_input, predictor_data = NAM_data,latitude_col_s = latitude_col_s,longitude_col_s = longitude_col_s, datum_col_s = datum_col_s,Dates_col_s = Dates_col_s, predictor_set_merged = "NAM_data") # join wrapper function
    rm(NAM_data)
  } else { # if (exists(NAM_data)) { # merge NAM data if there is any for this date - there wasn't any so just add columns with NA as space holder
    # read in file from Jan 1, 2008 to get the column names
      NAM_file_nameJan1_2008 <- file.path(ProcessedData.directory,NAM_folder,NAM_sub_folder,NAM_sub_sub_folder,paste("NAM_Step5_","2008-01-01","_batch",define_study_constants.fn("NAM_batch_date"),".csv",sep = "")) # get the NAM file for this_date
      NAM_data_step <- read.csv(file.path(NAM_file_nameJan1_2008),header=TRUE, stringsAsFactors = FALSE)  # load data file
    # change column names
      NAM_data_step <- replace_column_names.fn(df_in = NAM_data_step, old_col_name = "Lat", new_col_name = "Latitude") # replace "Lat" with "Latitude"
      NAM_data_step <- replace_column_names.fn(df_in = NAM_data_step, old_col_name = "Lon", new_col_name = "Longitude") # replace "Lat" with "Latitude"
      NAM_data_step <- replace_column_names.fn(df_in = NAM_data_step, old_col_name = "Local.Date", new_col_name = "Date") # replace "Lat" with "Latitude"
      # # recognize dates as dates
        #NAM_data_step[ , c(Dates_col_s)] <- as.Date(NAM_data_step[ , c(Dates_col_s)],"%Y-%m-%d") 
    # remove extraneous columns
    drop_cols <- c("Datum","Easting","Northing","old_lon","old_lat","old_Datum","State_FIPS","County_FIPS","Tract_code","ZCTA5_code") # define unnecessary columns
    NAM_data_step <- NAM_data_step[ , !(names(NAM_data_step) %in% drop_cols)] # drop unnecessary columns
    # track column names
    NAM_colnames_step <- colnames(NAM_data_step)
    NAM_colnames <- NAM_colnames_step[NAM_colnames_step %!in% c("Latitude","Longitude","Date")]
    rm(NAM_colnames_step)
  # add column as space holder if there was no data
    ML_input[ ,NAM_colnames] <- NA # add column as space holder if there was no data
  } # if (exists(NAM_data)) { # merge NAM data if there is any for this date
  return(ML_input) # output from function
} # end of merge_NAM_data.fn function

# Load and merge NED Data
merge_NED_data.fn <- function(ML_input, NED_file_name, ProcessedData.directory,predictor_sub_folder) {
  NED_data_list <- list()
  latitude_col_s <- "Latitude"
  longitude_col_s <- "Longitude"
  for (file_i in 1:length(NED_file_name)) { # Load and merge all NED Data files
    #print(paste("Processing",NED_file_name[file_i])) # COMMENT
    NED_data_step <- read.csv(file.path(ProcessedData.directory,predictor_sub_folder, NED_file_name[file_i]),header=TRUE) # load the file
    NED_data_step<- as.data.frame(NED_data_step)
    
    # change column names
    NED_data_step <- replace_column_names.fn(df_in = NED_data_step, old_col_name = "Lat", new_col_name = "Latitude") # replace "Lat" with "Latitude"
    NED_data_step <- replace_column_names.fn(df_in = NED_data_step, old_col_name = "Lon", new_col_name = "Longitude") # replace "Lat" with "Latitude"
    
    # remove extraneous columns
    drop_cols <- c("Datum","Easting","Northing","old_lon","old_lat","old_Datum", "State_FIPS","County_FIPS","Tract_code","ZCTA5_code")
    NED_data_step <- NED_data_step[ , !(names(NED_data_step) %in% drop_cols)]
    
    NED_data_list[[NED_file_name[file_i]]] <- NED_data_step
    rm(NED_data_step)
  } # for (file_i in 1:length(NED_file_name)) { # Load and merge all NED Data files
  NED_data_w_dups <- do.call("rbind", NED_data_list)
  #NED_data_w_dups$elevation <- round(NED_data_w_dups$elevation,5)
  NED_data_w_dups$elevation <- round(NED_data_w_dups$elevation)
  NED_data_w_dups$Latitude <- round(NED_data_w_dups$Latitude,5)
  NED_data_w_dups$Longitude <- round(NED_data_w_dups$Longitude,5)
  NED_data_step2 <- NED_data_w_dups[!duplicated(NED_data_w_dups),]
  
  which_NA <- which(is.na(NED_data_step2$Latitude) | is.na(NED_data_step2$Longitude))
  
  # round data to same value
  Loc_data <- NED_data_step2[ ,c("Latitude","Longitude")]
  Loc_data_repeats <- Loc_data[duplicated(Loc_data), ]
  if (dim(Loc_data_repeats)[1] > 0) { # only try to get rid of repeats if there are repeats
    for (repeat_i in 1:dim(Loc_data_repeats)[1]) { # cycle through locations that are repeated
      this_lat <- Loc_data_repeats[repeat_i,"Latitude"] # what is the latitude?
      this_lon <- Loc_data_repeats[repeat_i,"Longitude"] # what is the longitude?
      which_rows <- which(NED_data_step2$Latitude == this_lat & NED_data_step2$Longitude == this_lon) # find the rows with the repeated locations
      these_elevs <- NED_data_step2[which_rows,"elevation"]
      #print(these_elevs) # COMMENT
      if (max(these_elevs)-min(these_elevs) > 10) {stop(paste("different estimates of elevation for a location vary by more than 10 meters for date ",unique(ML_input$Date)))}
      NED_data_step2[which_rows,"elevation"] <- mean(these_elevs)
    }
  } # if (dim(Loc_data_repeats)[1] > 0) { # only try to get rid of repeats if there are repeats  
  NED_data <- NED_data_step2[!duplicated(NED_data_step2),]
  
  rm(NED_data_w_dups,NED_data_list,NED_data_step2)
  # join wrapper function
  ML_input <- merge_time_static_data.fn(ML_input_in = ML_input, predictor_data = NED_data,latitude_col_s = latitude_col_s,longitude_col_s = longitude_col_s, predictor_set_merged = "NED_data") 
  ML_input <- ML_input[!duplicated(ML_input), ] # get rid of repeated rows (not sure why they appear)
  
  if ("elevation" %!in% colnames(ML_input)) { # add column as space holder if there was no data
    ML_input$elevation <- NA # add column as space holder if there was no data
  } # add column as space holder if there was no data
  return(ML_input)
} # end of merge_NED_data.fn function

# Load and merge NLCD Data
merge_NLCD_data.fn <- function(buffer_radius, ML_input, NLCD_file_name,task_counter,ProcessedData.directory,predictor_sub_folder) {
  NLCD_data_list <- list()
  for (file_i in 1:length(NLCD_file_name)) { # Load and merge all NED Data files
    #print(paste("Processing NLCD file ",NLCD_file_name[file_i],sep = "")) # COMMENT
    NLCD_data_step <- read.csv(file.path(ProcessedData.directory,predictor_sub_folder, NLCD_file_name[file_i]),header=TRUE) # load the file
    NLCD_data_step <- as.data.frame(NLCD_data_step)
    latitude_col_s <- "Latitude"
    longitude_col_s <- "Longitude"
    
    # change column names
    NLCD_data_step <- replace_column_names.fn(df_in = NLCD_data_step, old_col_name = "Lat", new_col_name = "Latitude") # replace "Lat" with "Latitude"
    NLCD_data_step <- replace_column_names.fn(df_in = NLCD_data_step, old_col_name = "Lon", new_col_name = "Longitude") # replace "Lat" with "Latitude"
    var_col_name <- paste("NLCD_",buffer_radius,"_percent_urban_buffer",sep = "")
    #NLCD_data_step <- replace_column_names.fn(df_in = NLCD_data_step, old_col_name = "percent_urban_buffer", new_col_name = paste("NLCD_",buffer_radius,"_percent_urban_buffer",sep = "")) # input buffer radius into variable name
    NLCD_data_step <- replace_column_names.fn(df_in = NLCD_data_step, old_col_name = "percent_urban_buffer", new_col_name = var_col_name) # input buffer radius into variable name
    
    # remove extraneous columns
    drop_cols <- c("Datum","Easting","Northing","old_lon","old_lat","old_Datum", "State_FIPS","County_FIPS","Tract_code","ZCTA5_code","FID")
    NLCD_data_step <- NLCD_data_step[ , !(names(NLCD_data_step) %in% drop_cols)]
    
    NLCD_data_list[[NLCD_file_name[file_i]]] <- NLCD_data_step
    rm(NLCD_data_step)
  } # for (file_i in 1:length(NLCD_file_name)) { # Load and merge all NED Data files
  NLCD_data_w_dups <- do.call("rbind", NLCD_data_list) # un-list NLCD data
  NLCD_data <- average_slight_LatLon_variations.fn(Data_w_dups = NLCD_data_w_dups,var_col_name = var_col_name)
  
  # join wrapper function
  ML_input <- merge_time_static_data.fn(ML_input_in = ML_input, predictor_data = NLCD_data,latitude_col_s = latitude_col_s,longitude_col_s = longitude_col_s, predictor_set_merged = "NLCD_data") 
  ML_input <- ML_input[!duplicated(ML_input), ] # get rid of repeated rows (not sure why they appear)
  return(ML_input)
} # end of merge_NLCD_data.fn function

# Load and merge NDVI Data
merge_NDVI_data.fn <- function(ML_input,NDVI_file_name,ProcessedData.directory,predictor_sub_folder, this_Date) { #study_start_date, study_stop_date) {
  NDVI_data_list <- list() # create list for merging all data sets
  latitude_col_s <- "Latitude"
  longitude_col_s <- "Longitude"
  Dates_col_s <- "Date"
  for (file_i in 1:length(NDVI_file_name)) { # Load and merge all NDVI Data files
    print(paste("Processing",NDVI_file_name[file_i]," file_i =",file_i)) # COMMENT
    NDVI_data_step <- read.csv(file.path(ProcessedData.directory,predictor_sub_folder, NDVI_file_name[file_i]),header=TRUE, stringsAsFactors=FALSE) # load the AQS file
    NDVI_data_step <- as.data.frame(NDVI_data_step)
    date_format <- determine_date_format.fn(check_date = NDVI_data_step[10, c(Dates_col_s)]) # determine format used for this NDVI file; pick the 10th row as representative of the file (sometimes the first few rows are repeated headers)
    #print(date_format) # REMOVE
    NDVI_data_step[ , c(Dates_col_s)] <- as.Date(NDVI_data_step[ , c(Dates_col_s)],date_format) # recognize dates as dates
    
    #print(unique(NDVI_data_step$ndvi))
    
    #options(warn  =  1) # dont' throw an error when there's a warning and stop the code from running further
    #NDVI_data_step$ndvi <- as.numeric(NDVI_data_step$ndvi)
    #options(warn  =  2) # throw an error when there's a warning and stop the code from running further
    #print(unique(NDVI_data_step$ndvi))
    
    #print(paste("Earliest date in file:",min(NDVI_data_step[ , c(Dates_col_s)],na.rm = TRUE))) # COMMENT
    #plot.new()
    # #points(NDVI_data_step$Date, NDVI_data_step$NDVI_AOD)
    #plot(NDVI_data_step$Date, NDVI_data_step$NDVI_AOD)
    # change column names
    NDVI_data_step <- replace_column_names.fn(df_in = NDVI_data_step, old_col_name = "Lat", new_col_name = "Latitude") # replace "Lat" with "Latitude"
    NDVI_data_step <- replace_column_names.fn(df_in = NDVI_data_step, old_col_name = "Lon", new_col_name = "Longitude") # replace "Lat" with "Latitude"
    NDVI_data_step <- replace_column_names.fn(df_in = NDVI_data_step, old_col_name = "ndvi", new_col_name = "NDVI") # replace "Lat" with "Latitude"
    
    # remove extraneous columns
    drop_cols <- c("Datum","Easting","Northing","old_lon","old_lat","old_Datum","X","optional") # define unnecessary columns
    NDVI_data_step <- NDVI_data_step[ , !(names(NDVI_data_step) %in% drop_cols)] # drop unnecessary columns
    
    #print(unique(NDVI_data_step$ndvi))
    
    # isolate data for this date
    which_this_date <- which(NDVI_data_step[ , c(Dates_col_s)] == this_Date)
    if (length(which_this_date) > 0) { # is there any data for this date?
      print(paste("There is NDVI data for ",this_Date," in ",NDVI_file_name[file_i],sep = "")) # COMMENT
      NDVI_data_date <- NDVI_data_step[which_this_date, ] # isolate data for this date
      #print(unique(NDVI_data_date$ndvi))
      NDVI_data_date[ , c(latitude_col_s)] <- as.numeric(as.character(NDVI_data_date[ , c(latitude_col_s)])) # recognize latitude as numerical
      NDVI_data_date[ , c(longitude_col_s)] <- as.numeric(as.character(NDVI_data_date[ , c(longitude_col_s)])) # recognize longitude as numerical
      Check_data <- check_4_NAs.fn(no_NAs_allowed_cols = c("Latitude","Longitude","Date"), input_data = NDVI_data_date)
      if (length(Check_data)>0) {print("***Check_4_NAs.fn found questionable data. Investigate.***")}
      rm(Check_data)
      Check_data <- check_4_NAs.fn(no_NAs_allowed_cols = c("Latitude","Longitude","Date"), input_data = NDVI_data_date)
      if (length(Check_data)>0) {print("***Check_4_NAs.fn found questionable data. Investigate.***")}
      rm(Check_data)
      NDVI_data_date_no_repeats <- NDVI_data_date[!duplicated(NDVI_data_date), ]
      NDVI_data_list[[NDVI_file_name[file_i]]] <- NDVI_data_date_no_repeats # input data into list
      rm(NDVI_data_date,NDVI_data_date_no_repeats)
    } else { # if (length(which_this_date) > 0) { # is there any data for this date? - No
      #print(paste("No NDVI data for ",this_Date," in ",NDVI_file_name[file_i])) # COMMENT
      #  NDVI_data_date <- NDVI_data_step[20, ] # just grab 20th row as a place holder since nothing matches (preserve number of columns)
    } # if (length(which_this_date) > 0) { # is there any data for this date?  
    rm(NDVI_data_step) # clear variable
  } # for (file_i in 1:length(NDVI_file_name)) { # Load and merge all NDVI Data files
  NDVI_data_w_dups <- do.call("rbind", NDVI_data_list) # unlist data from various files
  NDVI_data <- NDVI_data_w_dups[!duplicated(NDVI_data_w_dups),] # de-duplicate rows of data
  
  #print(unique(NDVI_data$ndvi))
  
  if (!is.null(NDVI_data)) { # merge NDVI data if there is any for this date
    Check_data <- check_4_NAs.fn(no_NAs_allowed_cols = c("Latitude","Longitude","Date"), input_data = NDVI_data)
    if (length(Check_data)>0) {stop("***Check_4_NAs.fn found questionable data. Investigate.***")}
    rm(Check_data)
    if (class(NDVI_data$Date) != "Date") {stop("***class of Date_Local is not 'Date'. Investigate***")}
    if (class(NDVI_data$Latitude) != "numeric") {stop("***class of Date_Local is not 'Date'. Investigate***")}
    rm(NDVI_data_w_dups)
    ML_input <- merge_time_varying_data.fn(ML_input_in = ML_input, predictor_data = NDVI_data,latitude_col_s = latitude_col_s,longitude_col_s = longitude_col_s, datum_col_s = datum_col_s,Dates_col_s = Dates_col_s,predictor_set_merged = "NDVI_data") # join wrapper function
  } # if (!is.null(NDVI_data)) { # merge GASP data if there is any for this date
  rm(NDVI_data)
  
  # add column as space holder if there was no data
  if ("NDVI" %!in% colnames(ML_input)) { # add column as space holder if there was no data
    ML_input$NDVI <- NA # add column as space holder if there was no data
  } # add column as space holder if there was no data
  return(ML_input)
} # end of merge_NDVI_data.fn function

# average location-rounded values
average_slight_LatLon_variations.fn <- function(Data_w_dups,var_col_name) {
  #Data_w_dups <- NLCD_data_w_dups
  Data_w_dups$Latitude <- round(Data_w_dups$Latitude,5) # round location info
  Data_w_dups$Longitude <- round(Data_w_dups$Longitude,5) # round location info
  Data_step1 <- Data_w_dups[!duplicated(Data_w_dups), ] # remove any rows that are complete repeats
  rm(Data_w_dups) # clear variables
  Loc_data <- Data_step1[ ,c("Latitude","Longitude")]
  Data_output <- Loc_data[!duplicated(Loc_data), ]
  Data_output[ ,c(var_col_name)] <- NA
  for (loc_i in 1:dim(Data_output)[1]) {
    this_lat <- Data_output[loc_i, c("Latitude")]
    this_lon <- Data_output[loc_i, c("Longitude")]
    which_this_loc <- which(Data_step1$Latitude == this_lat & Data_step1$Longitude == this_lon)
    Data_output[loc_i,var_col_name] <- mean(Data_step1[which_this_loc,var_col_name])
    rm(this_lat,this_lon,which_this_loc)
  }
  rm(loc_i)
  return(Data_output)
}

# determine format of date
determine_date_format.fn <- function(check_date) {
  check_date_char <- as.character(check_date)
  if (!is.na(check_date)) { # can't check format on NA
  if (substr(check_date_char,(nchar(check_date_char)-4),(nchar(check_date_char)-4)) == "/" & 
      substr(check_date_char,(nchar(check_date_char)-2),(nchar(check_date_char)-2)) != "/") {
    date_format <- "%m/%d/%Y"
  } else if (substr(check_date_char,5,5) == "-") {
    date_format <- "%Y-%m-%d"
  } else if (substr(check_date_char,(nchar(check_date_char)-2),(nchar(check_date_char)-2)) == "/") {
    date_format <- "%m/%d/%y"
  } else if (substr(check_date_char,(nchar(check_date_char)-2),(nchar(check_date_char)-2)) == ":" &
             substr(check_date_char,(nchar(check_date_char)-10),(nchar(check_date_char)-10)) == "/") {
    date_format <- "m/d/Y H:M"
  } else if (substr(check_date_char,(nchar(check_date_char)),(nchar(check_date_char))) == "M" &
             substr(check_date_char,(nchar(check_date_char)-5),(nchar(check_date_char)-5)) == ":" & 
             (substr(check_date_char,2,2) == "/" | substr(check_date_char,3,3) == "/")) {
    date_format <- "%m/%d/%Y H:M:S !p"
  } else {
    stop("unknown date format - expand code in determine_date_format.fn in merging_data_functions.R")
  }
  } else{
    date_format <- NA
  }
  return(date_format)  
} # end of determine_date_format.fn function

'%!in%' <- function(x,y)!('%in%'(x,y)) # directly from https://stackoverflow.com/questions/5831794/opposite-of-in

# get data frame to have specified header, filling unused columns with NAs
harmonize_column_names.fn <- function(df_in, goal_header) {
  df_out <- data.frame(matrix(data = NA, nrow = dim(df_in)[1], ncol = length(goal_header)))
  names(df_out) <- goal_header
  for (x in 1:length(goal_header)) { # fill in each column in df_out
    this_goal_column <- goal_header[x]
    if (this_goal_column %in% names(df_in)) { # was this column in df_in? If so, fill in data.
      df_out[ , this_goal_column] <- df_in[ , this_goal_column]
    } # if (this_goal_column %in% names(df_in)) { # was this column in df_in? If so, fill in data.
  } # for (x in 1:length(goal_header)) { # fill in each column in df_out
  return(df_out)
} # end of harmonize_column_names.fn function
